X-Git-Url: http://git.vanrenterghem.biz/git.ikiwiki.info.git/blobdiff_plain/d0b1b244281102a07f675880c401ca831ff113d4..80f2edffc0283d1e6074d9a18676795d35e6f4e1:/IkiWiki/Plugin/map.pm?ds=sidebyside

diff --git a/IkiWiki/Plugin/map.pm b/IkiWiki/Plugin/map.pm
index 93ff3194a..328493116 100644
--- a/IkiWiki/Plugin/map.pm
+++ b/IkiWiki/Plugin/map.pm
@@ -9,13 +9,22 @@ package IkiWiki::Plugin::map;
 
 use warnings;
 use strict;
-use IkiWiki 2.00;
+use IkiWiki 3.00;
 
-sub import { #{{{
+sub import {
+	hook(type => "getsetup", id => "map", call => \&getsetup);
 	hook(type => "preprocess", id => "map", call => \&preprocess);
-} # }}}
+}
 
-sub preprocess (@) { #{{{
+sub getsetup () {
+	return
+		plugin => {
+			safe => 1,
+			rebuild => undef,
+		},
+}
+
+sub preprocess (@) {
 	my %params=@_;
 	$params{pages}="*" unless defined $params{pages};
 	
@@ -25,8 +34,14 @@ sub preprocess (@) { #{{{
 	my %mapitems;
 	foreach my $page (keys %pagesources) {
 		if (pagespec_match($page, $params{pages}, location => $params{page})) {
-			$mapitems{$page}=1;
-
+			if (exists $params{show} && 
+			    exists $pagestate{$page} &&
+			    exists $pagestate{$page}{meta}{$params{show}}) {
+				$mapitems{$page}=$pagestate{$page}{meta}{$params{show}};
+			}
+			else {
+				$mapitems{$page}='';
+			}
 			# Check for a common prefix.
 			if (! defined $common_prefix) {
 				$common_prefix=$page;
@@ -37,6 +52,9 @@ sub preprocess (@) { #{{{
 				my @b=split(/\//, $common_prefix);
 				$common_prefix="";
 				while (@a && @b && $a[0] eq $b[0]) {
+					if (length $common_prefix) {
+						$common_prefix.="/";
+					}
 					$common_prefix.=shift(@a);
 					shift @b;
 				}
@@ -45,12 +63,14 @@ sub preprocess (@) { #{{{
 	}
 	
 	# Common prefix should not be a page in the map.
-	while (length $common_prefix && exists $mapitems{$common_prefix}) {
+	while (defined $common_prefix && length $common_prefix &&
+	       exists $mapitems{$common_prefix}) {
 		$common_prefix=IkiWiki::dirname($common_prefix);
 	}
 
 	# Needs to update whenever a page is added or removed, so
 	# register a dependency.
 	add_depends($params{page}, $params{pages});
 	# Explicitly add all currently shown pages, to detect when pages
 	# are removed.
@@ -60,13 +80,18 @@ sub preprocess (@) { #{{{
 	my $parent="";
 	my $indent=0;
 	my $openli=0;
+	my $addparent="";
 	my $map = "<div class='map'>\n<ul>\n";
 	foreach my $item (sort keys %mapitems) {
-		$item=~s/^\Q$common_prefix\E\/// if length $common_prefix;
+		my @linktext = (length $mapitems{$item} ? (linktext => $mapitems{$item}) : ());
+		$item=~s/^\Q$common_prefix\E\///
+			if defined $common_prefix && length $common_prefix;
 		my $depth = ($item =~ tr/\//\//) + 1;
 		my $baseitem=IkiWiki::dirname($item);
 		while (length $parent && length $baseitem && $baseitem !~ /^\Q$parent\E(\/|$)/) {
 			$parent=IkiWiki::dirname($parent);
+			last if length $addparent && $baseitem =~ /^\Q$addparent\E(\/|$)/;
+			$addparent="";
 			$indent--;
 			$map .= "</li>\n";
 			if ($indent > 0) {
@@ -90,8 +115,12 @@ sub preprocess (@) { #{{{
 			}
 			if ($depth > $indent) {
 				$p.="/".shift(@bits);
+				$addparent=$p;
+				$addparent=~s/^\///;
 				$map .= "<li>"
-					.htmllink($params{page}, $params{destpage}, $p, class => "mapparent")
+					.htmllink($params{page}, $params{destpage},
+						 "/".$common_prefix.$p, class => "mapparent",
+						 noimageinline => 1)
 					."\n";
 				$openli=1;
 			}
@@ -102,7 +131,9 @@ sub preprocess (@) { #{{{
 		$map .= "</li>\n" if $openli;
 		$map .= "<li>"
 			.htmllink($params{page}, $params{destpage}, 
-				"/".$common_prefix."/".$item, class => "mapitem")
+				"/".$common_prefix."/".$item,
+				@linktext,
+				class => "mapitem", noimageinline => 1)
 			."\n";
 		$openli=1;
 		$parent=$item;
@@ -113,6 +144,6 @@ sub preprocess (@) { #{{{
 	}
 	$map .= "</div>\n";
 	return $map;
-} # }}}
+}
 
 1
@@ -45,12 +63,14 @@ sub preprocess (@) { #{{{
 	}
 
-	# Needs to update whenever a page is added or removed, so
-	# register a dependency.
+	# Needs to update whenever a page is added or removed (or in some
+	# cases, when its content changes, if show=title), so register a
+	# dependency.
 	add_depends($params{page}, $params{pages});
 	# Explicitly add all currently shown pages, to detect when pages